package choco;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.*;
import org.chocosolver.solver.variables.IntVar;

public class send_mory {
	public static void main(String[] args) {
		
		Model model = new Model("send + more = money");
	// Declaration of variables
		IntVar s = model.intVar("S", 0, 9);
		IntVar e = model.intVar("E", 0, 9);
		IntVar n = model.intVar("N", 0, 9);
		IntVar d = model.intVar("D", 0, 9);
		IntVar m = model.intVar("M", 0, 9);
		IntVar o = model.intVar("O", 0, 9);
		IntVar r = model.intVar("R", 0, 9);
		IntVar y = model.intVar("Y", 0, 9);

	// Specification of constraints
	// no leading zeros
		model.arithm(m, ">", 0).post();
		model.arithm(s, ">", 0).post();
		
	// all digits are different
		model.allDifferent(new IntVar[]{s, e, n, d, m, o, r, y}).post();
		
	// sum must be correct
	// clumsy syntax
		IntVar op1 = model.intVar("S", 0, 10000);
		IntVar op2 = model.intVar("S", 0, 10000);
		IntVar res = model.intVar("S", 0, 100000);
		
		model.arithm(res, "=", op1, "+", op2).post();
		
		// op1 = 1000s + 100e +10n + d
		op1.eq(s.mul(1000).add(e.mul(100)).add(n.mul(10)).add(d)).post();
		// op2 = 1000m + 100o +10r + e
		op2.eq(m.mul(1000).add(o.mul(100)).add(r.mul(10)).add(e)).post();
		// res = 10000m + 1000o +100n + 10e +y
		res.eq(m.mul(10000).add(o.mul(1000)).add(n.mul(100)).add(e.mul(10)).add(y)).post();
		
	// alternative syntax
	/*
		IntVar[] ALL = new IntVar[]{
		    s, e, n, d,
		    m, o, r, e,
		    m, o, n, e, y};
		int[] COEFFS = new int[]{
		    1000, 100, 10, 1,
		    1000, 100, 10, 1,
		    -10000, -1000, -100, -10, -1};
		model.scalar(ALL, COEFFS, "=", 0).post();
	*/	
		// Execute and show Results
		Solver slv = model.getSolver();
		if (slv.solve()){
			//pretty output
				System.out.println(" " + Integer.toString(1000*s.getValue()+ 100*e.getValue()+10*n.getValue()+d.getValue()));
				System.out.println("+" + Integer.toString(1000*m.getValue()+ 100*o.getValue()+10*r.getValue()+e.getValue()));
				System.out.println("-----");
				System.out.println(10000*m.getValue()+1000*o.getValue()+ 100*n.getValue()+10*e.getValue()+y.getValue());
			// show normal output
				slv.showStatistics();
				slv.showSolutions();
				slv.findSolution();
		} else {
			System.out.println("no solutions");
		}
	}
}	